/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <msp430.h>
#include "uart.h"

#define CMD_RESET       0x06
#define CMD_START       0x08
#define CMD_POWERDOWN   0x02
#define CMD_RDATA       0x10
#define CMD_RREG        0x20
#define CMD_WREG        0x40

#define SYNC            0x55

long *ads112_raw = 0;
long *rtd_raw = 0;
long *temp_raw = 0;
long *tmp235_raw = 0;
long *lmt70_raw = 0;

const int lmt70_lut[] = { //150 - -50C in 10 deg/step
		  3027, 3581, 4127, 4668, 5206, 5741, 6275, 6806, 7336, 7864, 8389,
		  8912, 9432, 9950, 10466, 11491, 11999, 12504, 13006, 13504};

void ads112_init(void){
    uart0_init();

    P1DIR |= (1<<4); // Reset
    P1OUT &= ~(1<<4);
    __delay_cycles(10); //min 250ns
    P1OUT |= (1<<4);
    __delay_cycles(8*80); //min 80us
}

static void ads112_write_reg(char reg, char data){
    uart0_putc(SYNC);
    uart0_putc(CMD_WREG|(reg<<1));
    uart0_putc(data);
}

static char ads112_read_reg(char reg){
    uart0_putc(SYNC);
    uart0_putc(CMD_RREG|(reg<<1));
    while(!uart0_data_available());
    return uart0_getc();
}

static short ads112_read_data(void){
    int temp;
    uart0_putc(SYNC);
    uart0_putc(CMD_RDATA);
    while(!uart0_data_available());
    temp = uart0_getc();
    while(!uart0_data_available());
    return uart0_getc()<<8 | temp;
}

int ads112_getoffset(void){
#if 1

	int reg = ads112_read_reg(0);
	int offset;
    ads112_write_reg(0, 0xe0|reg&0x0f);  // AINp = AINn = avdd/2

    uart0_putc(SYNC);
    uart0_putc(CMD_START);      // start conversation

    while(!(ads112_read_reg(2) & 0x80));    // wait for conversion to finish
    offset = ads112_read_data();
    ads112_write_reg(0, reg);

#else
    int offset = 0;
#endif
    return offset;
}

long ads112_gettc(void){
	long temp;
    ads112_write_reg(0, 0x0a);  // AINp = AIN0, AINn = AIN1, gain = 32, PGA on
    ads112_write_reg(1, 0x00);  // DR = 20, single mode, internal ref
    ads112_write_reg(2, 0x00);  // data counter disabled, crc disabled, burn out disabled, idac off
    ads112_write_reg(3, 0x00);  // idac disabled
    ads112_write_reg(4, 0x00);  // gpios input, not used
    int offset = ads112_getoffset();

    uart0_putc(SYNC);
    uart0_putc(CMD_START);      // start conversation

    while(!(ads112_read_reg(2) & 0x80));    // wait for conversion to finish
    temp = ads112_read_data();              // return conversation value
    *ads112_raw = temp - offset;

    temp *= (2048000/32);
    temp /= 32768;
    return temp;
}

long ads112_gettemp(void){
    long temp;
    ads112_write_reg(0, 0x00);  // AINp = AIN0, AINn = AIN1, gain = 1, PGA on
    ads112_write_reg(1, 0x01);  // DR = 20, single mode, internal ref, temp mode
    ads112_write_reg(2, 0x00);  // data counter disabled, crc disabled, burn out disabled, idac off
    ads112_write_reg(3, 0x00);  // idac disabled
    ads112_write_reg(4, 0x00);  // gpios input, not used
    int offset = ads112_getoffset();

    uart0_putc(SYNC);
    uart0_putc(CMD_START);      // start conversation

    while(!(ads112_read_reg(2) & 0x80));    // wait for conversion to finish
    temp = ads112_read_data();              // return conversation value
    *temp_raw = temp - offset;

    temp = temp * 3.125;
    temp >>= 2;
    return temp; // in C * 100
    //code / 4 * 0.03125 = temp in C
}

long ads112_getrtd(void){
    long temp;
    ads112_write_reg(0, 0x7a);  // AINp = AIN3, AINn = AIN2, gain = 32, PGA on
    ads112_write_reg(1, 0x02);  // DR = 20, single mode, ext ref
    ads112_write_reg(2, 0x04);  // data counter disabled, crc disabled, burn out disabled, idac 250uA
    ads112_write_reg(3, 0x70);  // idac on AIN2 and 3
    ads112_write_reg(4, 0x00);  // gpios input, not used
    int offset = ads112_getoffset();

    uart0_putc(SYNC);
    uart0_putc(CMD_START);      // start conversation

    while(!(ads112_read_reg(2) & 0x80));    // wait for conversion to finish
    temp = ads112_read_data()-offset;       // return conversation value
    *rtd_raw = temp;

    ads112_write_reg(3, 0x00);  // idac disabled

    temp *= 810/2; // RRef / Gain / 2
    temp >>= 8;
    temp *= 100;
    temp >>= 7;
    temp = (temp - 10000) / 0.385; //todo: use table

    return temp; // in C * 100
    // (code * 3240) / (32768 * Gain/2) = R PT100
}

long ads112_getlmt70(void){
	long temp;
	int index;
	int i;
	int slope;
	int base;
	ads112_write_reg(0, 0xa0);  // AINp = AIN2, AINn = AVss, gain = 1, PGA off
    ads112_write_reg(1, 0x00);  // DR = 20, single mode, internal ref
    ads112_write_reg(2, 0x00);  // data counter disabled, crc disabled, burn out disabled, idac off
    ads112_write_reg(3, 0x00);  // idac disabled
    ads112_write_reg(4, 0x00);  // gpios input, not used
    int offset = ads112_getoffset();

    uart0_putc(SYNC);
	uart0_putc(CMD_START);      // start conversation

	while (!(ads112_read_reg(2) & 0x80));    // wait for conversion to finish
	temp = ads112_read_data() - offset;              // return conversation value
	*lmt70_raw = temp;

	temp *= 20480;
	temp /= 32768; // in mV*10

	for(i=0;i<sizeof(lmt70_lut)/sizeof(lmt70_lut[0]);i++){
		if(lmt70_lut[i] > temp){
			index = i;
			break;
		}
	}
	slope = lmt70_lut[index] - lmt70_lut[index-1];
	base = 150 - index * 10;
	temp = lmt70_lut[index] - temp;
	temp = temp*1000 / slope;
	temp += base*100;

	return temp;
}

long ads112_gettmp235(void){
	long temp;
	ads112_write_reg(0, 0xb0);  // AINp = AIN2, AINn = AVss, gain = 1, PGA off
    ads112_write_reg(1, 0x00);  // DR = 20, single mode, internal ref
    ads112_write_reg(2, 0x00);  // data counter disabled, crc disabled, burn out disabled, idac off
    ads112_write_reg(3, 0x00);  // idac disabled
    ads112_write_reg(4, 0x00);  // gpios input, not used
    int offset = ads112_getoffset();

    uart0_putc(SYNC);
	uart0_putc(CMD_START);      // start conversation

	while (!(ads112_read_reg(2) & 0x80));    // wait for conversion to finish
	temp = ads112_read_data() - offset;              // return conversation value
	*tmp235_raw = temp;

	temp *= 20480;
	temp /= 32768; // in mV*10
	temp -= 5000;

	return temp;
}
