/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include "../main.h"

#include "uart.h"
#include "vt100.h"
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#define ESC 		"\033"
#define CSI 		ESC"["
#define UNDERLINE	CSI"4m"
#define RESETFORMAT	CSI"0m"
#define CURSOROFF	CSI"?25l"
#define CURSORON	CSI"?25h"

#define UNDERLINECHAR(x)	UNDERLINE x RESETFORMAT

#define TABSTART	25
#define TABSTART2	45
#define TABSTART3	65
#define TABSTART4	80
#define VALUESTART	8

const struct menu_struct menu[]={
	{"Config", 0, 1, menu_int, &config_id},
	{"ADC", 0, 2, menu_str, &adc_name},         {"Raw", TABSTART, 2, menu_long, &adc_raw},		{"uV", TABSTART2, 2, menu_long, &adc_value},			{"dTemp", TABSTART3, 2, menu_int, &tc_temp},
	{"CJC", 0, 3, menu_str, &cjc_name[0]},		{"Raw", TABSTART, 3, menu_int, &cjc_raw[0]},	{"aTemp", TABSTART2, 3, menu_int, &cjc_value[0]},	{"uV", TABSTART3, 3, menu_int, &cjc_volt[0]},	{"Temp", TABSTART4, 3, menu_int, &cjc_temp[0]},
	{"", 0, 4, menu_str, &cjc_name[1]},			{"Raw", TABSTART, 4, menu_int, &cjc_raw[1]},	{"aTemp", TABSTART2, 4, menu_int, &cjc_value[1]},	{"uV", TABSTART3, 4, menu_int, &cjc_volt[1]},	{"Temp", TABSTART4, 4, menu_int, &cjc_temp[1]},
	{"", 0, 5, menu_str, &cjc_name[2]},			{"Raw", TABSTART, 5, menu_int, &cjc_raw[2]},	{"aTemp", TABSTART2, 5, menu_int, &cjc_value[2]},	{"uV", TABSTART3, 5, menu_int, &cjc_volt[2]},	{"Temp", TABSTART4, 5, menu_int, &cjc_temp[2]},

};

const enum field_enum update_field[] = {2, 3, 4,
										6, 7, 8, 9,
										11, 12, 13, 14,
										16, 17, 18, 19};

static int edit_flag=0;
static volatile int write_flag=0;
/**
 * Initialize terminal as vt100 terminal
 */
void vt100_init(void){
	uart_printf(CSI"37;40m" CSI"2J" CSI"H" CSI"6n" CSI"2J");
	uart_printf(CURSOROFF);	//cursor off
}

/**
 * Set cursor to position
 * @param x x coordinate
 * @param y y coordinate
 */
void vt100_setcur(int x, int y){
	uart_printf(CSI"%d;%dH", y, x);
}

/**
 * Printf over UART to specific position
 * @param x position
 * @param y position
 * @param format format to print
 */
void vt100_printf(int x, int y, const char * format, ...){
	char buffer[64];
	va_list args;
	va_start(args, format);
	vsprintf(buffer,format, args);
	vt100_setcur(x, y);
	uart_printf(buffer);
	va_end(args);
}

/**
 * Initialize menu on serial line by writing every field
 */
void menu_init(void){
	int i;
	vt100_init();
	for(i=0; i<(sizeof(menu)/sizeof(menu[0])); i++){
		menu_write_field((enum field_enum)i);
	}
}

/**
 * Edit one field by setting the cursor to the position
 * @param field field to edit
 */
void menu_edit(enum field_enum field){
	edit_flag = 1;
	vt100_setcur(menu[field].startx+VALUESTART, menu[field].starty);
	uart_printf(CURSORON);
}

void menu_print_value(enum field_enum field) {
	switch (menu[field].type) {
		case menu_int:
			vt100_printf(menu[field].startx + VALUESTART, menu[field].starty, "%-6d", *((int*) menu[field].ptr));
			break;
		case menu_long:
			vt100_printf(menu[field].startx + VALUESTART, menu[field].starty, "%-6ld", *((long*) menu[field].ptr));
			break;
		case menu_int_ptr:
		    vt100_printf(menu[field].startx + VALUESTART, menu[field].starty, "%-5d", **((int**) menu[field].ptr));
		    break;
		case menu_str:
			vt100_printf(menu[field].startx + VALUESTART, menu[field].starty, "%s", (char*) menu[field].ptr);
			break;

	}
}

/**
 * Write the value of a field to console
 * @param field field to update
 */
void menu_write_field(enum field_enum field){
	//if(menu_init_state < 2) return;
	if(!edit_flag){
		write_flag = 1;
		vt100_printf(menu[field].startx, menu[field].starty, menu[field].text);
		menu_print_value(field);
		write_flag = 0;
	}
}

/**
 * Updates the value of a field
 * @param field field to update
 */
void menu_update_field(enum field_enum field){
	//if(menu_init_state < 2) return;
	if(!edit_flag){
		write_flag = 1;
		menu_print_value(field);
		write_flag = 0;
	}
}

void menu_update_all(void){
	int i;
	for(i=0; i<sizeof(update_field)/sizeof(update_field[0]);i++){
		menu_update_field(update_field[i]);
	}
}

void menu_input(char c){
	switch(c){
		case 'v':
			terminal_log ^= 1;
			break;
	}
}

